wrap library

Classes

InsertableWrapParentData
Parent data for use with InsertableWrapRender.
InsertableWrapRender
Displays its children in multiple horizontal or vertical runs.
InsertionPoint
a position at which a child can be inserted
IWrap
A layout widget that displays its children in multiple horizontal or vertical runs. A version of the Wrap with a cleaner layout implementation and an 'insertionIndexAt' method for drag and drop insertion.

Functions

distributeWrapSpace(WrapAlignment alignment, double freeSpace, double itemSpacing, int itemCount, bool flipped) → (double, double)
flipOffset(Offset o) Offset
flipSize(Size s) Size
insertionOf(GlobalKey<State<StatefulWidget>> key, Offset globalOffset) InsertionPoint
gets the insertion point relative to globalOffset, necessary for rearranging order through drag and drop.
wrapAlignmentFlippsed(WrapCrossAlignment v) → dynamic
wrapCrossAlignmentAlignment(WrapCrossAlignment v) → dynamic