inserterWide property
is wide when it's not inserting at a between point, in which case a narrow selector visual wouldn't look right
Implementation
final bool inserterWide;
is wide when it's not inserting at a between point, in which case a narrow selector visual wouldn't look right
final bool inserterWide;