position property
relative to the wrap (note, you use a global position to get the insertionPoint, they're different coordinate systems)
Implementation
final Offset position;
relative to the wrap (note, you use a global position to get the insertionPoint, they're different coordinate systems)
final Offset position;